function [networkMatrix,Y] = plotNetworkMatrix_DLS_CV(target_gene_agis, all_genes_agis, cv_results, conf_th, show_confs, show_plot, GenesAliases, networkMatrix, Y)
% [networkMatrix,genesNames,Y,bg] = plotNetworkMatrix_DLS_CV(target_gene_agis, all_genes_agis, cv_results, conf_th, show_confs, GenesAliases, networkMatrix, Y);
%
% Crea un grafo que muestra los DLSs que predijeron a los target genes en
% el cross validation.
% networkMatrix: sparse matrix where classNetworkMatrix(i,j) = 1 si
% link desde nodo i hasta nodo j.

    n_total_genes = numel(all_genes_agis);
    n_target_genes = numel(target_gene_agis);
    if n_target_genes == 0
        target_gene_agis = cv_results.genes_agis;
        n_target_genes = numel(target_gene_agis);
    end
    
    if ~exist('Y','var')
        Y = zeros(n_total_genes,1);
        [~,idx1] = ismember(target_gene_agis,cv_results.genes_agis);
        [~,idx2] = ismember(target_gene_agis,all_genes_agis);
        Y(idx2) = cv_results.Y(idx1);
    else
        if numel(Y) ~= n_total_genes
            error('Y size does not match all_genes_agis size.')
        end
    end
    
    % generate networkMatrix
    if ~exist('networkMatrix','var')
        networkMatrix = sparse(n_total_genes,n_total_genes);
    else
        if size(networkMatrix,1) ~= n_total_genes
            error('networkMatrix size does not match all_genes_agis size.')
        end
    end
    for i = 1:n_target_genes
        
        % find target gene index
        target_idx = find(strcmp(target_gene_agis{i},all_genes_agis));
        
        % find the cv fold in which the target gene was classified
        for fold = 1:cv_results.nfolds
            fold_agis = cv_results.genes_agis(cv_results.te_idxs{fold});
            if ismember(target_gene_agis{i},fold_agis)
                break;
            end
        end
        target_cl_mask = strcmp(fold_agis,target_gene_agis{i});
        
        % check for each dls's if predicted the target gene
        for j = 1:numel(cv_results.test.cl_results{fold}.dls)
            conf = cv_results.test.cl_results{fold}.dls{j}.Confs(target_cl_mask);
            if conf >= conf_th
                dls_idx = find(strcmp(cv_results.training.tr_results{fold}.genes_agis(cv_results.test.cl_results{fold}.dls{j}.dls_idx),all_genes_agis));
                networkMatrix(dls_idx,target_idx) = max(networkMatrix(dls_idx,target_idx),conf);
                if(Y(target_idx) ~= 2)
                    Y(target_idx) = cv_results.Y(i);
                end
                Y(dls_idx) = 2;
            end
        end
        
    end
    
    if show_plot
        viewNetwork_DLS(networkMatrix,all_genes_agis,show_confs,Y,GenesAliases);
    end
    
end
