function [networkMatrices,Y] = plotNetworkMatrix_DLS_target(target_agis, all_genes_agis, cl_results, tr_results, conf_th, show_confs, show_plot, GenesAliases, networkMatrices)
% [networkMatrix,genesNames,Y,bg] = plotNetworkMatrix_DLS_cl_target(target_agis, all_genes_agis, cl_results, conf_th, show_confs, GenesAliases, networkMatrix, Y);
%
% Crea un grafo que muestra los DLSs que predijeron a los target genes en
% el cross validation.
% networkMatrix: sparse matrix where classNetworkMatrix(i,j) = 1 si
% link desde nodo i hasta nodo j.

    n_total_genes = numel(all_genes_agis);
    n_target_genes = numel(target_agis);
    if n_target_genes == 0
        target_agis = cl_results.preds.genes_agis;
        n_target_genes = numel(target_agis);
    end
    
    % generate networkMatrix
    if ~exist('networkMatrices','var')
        networkMatrices.confs = sparse(n_total_genes,n_total_genes);
        networkMatrices.coexps = sparse(n_total_genes,n_total_genes);
        networkMatrices.scores = sparse(n_total_genes,n_total_genes);
        Y = zeros(n_total_genes,1);
    else
        if size(networkMatrices.confs,1) ~= n_total_genes
            error('networkMatrices size does not match all_genes_agis size.')
        end
        Y = networkMatrices.Y;
        if numel(Y) ~= n_total_genes
            error('Y size does not match all_genes_agis size.')
        end
    end
    
    networkMatrices.all_genes_agis = all_genes_agis;
    networkMatrices.target_agis = target_agis;
    
    for i = 1:n_target_genes
        
        % find target gene index in cl_genes
        target_idx = find(strcmp(target_agis{i},all_genes_agis));
        target_cl_mask = strcmp(cl_results.preds.genes_agis,target_agis{i});
        
        % check for each dls's if predicted the target gene
        valid_dls_idxs = find(tr_results.valid_dls_mask);
        for j = 1:tr_results.n_valid_dls
            dls_idx = valid_dls_idxs(j);
            conf = cl_results.preds.Confs_matrix(dls_idx,target_cl_mask);
            if conf >= conf_th
                dls_abs_idx = find(strcmp(tr_results.genes_agis(tr_results.dls_idxs(dls_idx)),all_genes_agis));
                networkMatrices.confs(dls_abs_idx,target_idx) = conf;
                networkMatrices.coexps(dls_abs_idx,target_idx) = cl_results.preds.Coexps_matrix(dls_idx,target_cl_mask);
                networkMatrices.scores(dls_abs_idx,target_idx) = cl_results.preds.Scores_matrix(dls_idx,target_cl_mask);
            end
        end
        
        % check if target predicted the other genes as DLSs
        target_dls_idx = find(ismember(tr_results.genes_agis(tr_results.dls_idxs),target_agis{i}));
        is_valid = tr_results.valid_dls_mask(target_dls_idx);
        if is_valid
            confs = cl_results.preds.Confs_matrix(target_dls_idx,:);
            preds_mask = confs >= conf_th;
            preds_agis = cl_results.preds.genes_agis(preds_mask);
            [~,preds_idxs] = ismember(preds_agis,all_genes_agis);
            networkMatrices.confs(target_idx,preds_idxs) = confs(preds_mask);
            networkMatrices.coexps(target_idx,preds_idxs) = cl_results.preds.Coexps_matrix(target_dls_idx,preds_mask);
            networkMatrices.scores(target_idx,preds_idxs) = cl_results.preds.Scores_matrix(target_dls_idx,preds_mask);
        end
        
    end
    Y(ismember(all_genes_agis,cl_results.preds.genes_agis)) = 2;
    Y(ismember(all_genes_agis,tr_results.genes_agis(tr_results.Y_tr == 1))) = 1;
    Y(ismember(all_genes_agis,tr_results.genes_agis(tr_results.Y_tr == -1))) = -1;
    Y(ismember(all_genes_agis,tr_results.genes_agis(tr_results.dls_idxs(tr_results.valid_dls_mask)))) = 3;
    networkMatrices.Y = Y;
    
    if show_plot && show_confs == 1
        viewNetwork_DLS(networkMatrices.confs,all_genes_agis,true,Y,GenesAliases);
    elseif show_plot && show_confs == 2
        viewNetwork_DLS(networkMatrices.coexps,all_genes_agis,true,Y,GenesAliases);
    elseif show_plot && show_confs == 0
        viewNetwork_DLS(networkMatrices.coexps,all_genes_agis,false,Y,GenesAliases);
    end
    
end
