function [networkMatrix,genesNames,Y,bg] = plotNetworkMatrix_DLS_tr_target(tr_target_agis, all_genes_agis, tr_results, conf_th, show_confs, GenesAliases, networkMatrix, Y)
% [networkMatrix,genesNames,Y,bg] = plotNetworkMatrix_DLS_tr_target(tr_target_agis, all_genes_agis, tr_results, conf_th, show_confs, GenesAliases, networkMatrix, Y);
%
% Crea un grafo que muestra los DLSs que predijeron a los target genes en
% el cross validation.
% networkMatrix: sparse matrix where classNetworkMatrix(i,j) = 1 si
% link desde nodo i hasta nodo j.

    n_total_genes = numel(all_genes_agis);
    n_target_genes = numel(tr_target_agis);
    if n_target_genes == 0
        tr_target_agis = tr_results.genes_agis;
        n_target_genes = numel(tr_target_agis);
    end
    
    if ~exist('Y','var')
        Y = zeros(n_genes,1);
    else
        if numel(Y) ~= n_total_genes
            error('Y size does not match all_genes_agis size.')
        end
    end
    
    % generate networkMatrix
    if exist('networkMatrix','var')
        if size(networkMatrix,1) ~= n_total_genes
            error('networkMatrix size does not match all_genes_agis size.')
        end
    else
        networkMatrix = sparse(n_total_genes,n_total_genes);
    end
    
    target_idxs = zeros(n_target_genes,1);
    for i = 1:n_target_genes
        
        % find target gene index in cl_genes
        target_idxs(i) = find(strcmp(tr_target_agis{i},all_genes_agis));
        
        % check for each dls's if predicted the target gene
        for j = 1:numel(tr_results.dls)
            conf = tr_results.dls{j}.Confs(target_idxs(i));
            if conf >= conf_th
                dls_idx = tr_results.dls{j}.dls_idx;
                networkMatrix(dls_idx,target_idx) = max(networkMatrix(dls_idx,target_idx),conf);
                Y(dls_idx) = 2;
            end
        end
        
    end
    Y(target_idxs) = 1;
    
    % get genes names
    if exist('GenesAliases','var')
        genesNames = getGenesNames(all_genes_agis,GenesAliases);
    else
        genesNames = all_genes_agis;
    end
    
    if numel(unique(genesNames)) ~= numel(genesNames)
        genesNames = all_genes_agis;
    end
    
    bg = viewNetwork_DLS(networkMatrix,genesNames,show_confs,Y);
    
end
