function cl_results = DLSM_classify(X_new,new_genes_agis,tr_results,sigma,parallel,conf_th,min_coexp,min_dls_preds,is_training)
    
    if ~exist('is_training','var')
        is_training = false;
    end
    if ~exist('conf_th','var')
        conf_th = 0.5;
    end
    
    opened = false;
    
    if parallel && matlabpool('size') == 0
        matlabpool('open');
        opened = true;
    end
    
    if ~parallel && matlabpool('size') > 0
        matlabpool close force local
    end
    
    cl_results.params.sigma = sigma;
    cl_results.params.min_coexp = min_coexp;
    cl_results.params.conf_th = conf_th;
    min_dls_preds = max(min_dls_preds,1);
    cl_results.params.min_dls_preds = min_dls_preds;
    
    n_dls = tr_results.n_dls;
    n_samples = size(X_new,1);
    
    
    Coexps = zeros(n_dls,n_samples);
    Confs = zeros(n_dls,n_samples);
    Scores = zeros(n_dls,n_samples);
    
    % iterate over each DLS
    tr_dls = tr_results.dls;

    for i = 1:n_dls
        
        if tr_dls{i}.is_valid

            [confs_temps,Coexps(i,:)] = Bayes_confidence(tr_dls{i},X_new,sigma,min_coexp);
            if is_training
                confs_temps(tr_dls{i}.dls_idx) = min(confs_temps);
            end
            Confs(i,:) = confs_temps;
            Scores(i,:) = (confs_temps.*abs(Coexps(i,:))).*numel(tr_dls{i}.feats);
        else
            
        end
        
    end
    
    if opened && parallel && matlabpool('size') > 0
        matlabpool close force local
    end
    
    cl_results.n_samples = n_samples;
    cl_results.n_dls = n_dls;
    cl_results.n_valid_dls = tr_results.n_valid_dls;
    cl_results.valid_dls_mask = tr_results.valid_dls_mask;
    cl_results.preds.dls_scores = tr_results.dls_scores;
    cl_results.preds.dls_n_feats = tr_results.dls_n_feats;
    cl_results.preds.dls_agis = tr_results.genes_agis(tr_results.dls_idxs);
    cl_results.preds.genes_agis = new_genes_agis;
    cl_results.preds.Confs_matrix = Confs;
    cl_results.preds.Scores_matrix = Scores;
    cl_results.preds.Coexps_matrix = Coexps;
    
    [cl_results.preds.max_confs,cl_results.preds.max_confs_dls_local_idxs] = max(Confs,[],1);
    cl_results.preds.max_confs_dls_agis = cl_results.preds.dls_agis(cl_results.preds.max_confs_dls_local_idxs);
    [temp,cl_results.preds.max_confs_dls_idxs] = ismember(cl_results.preds.max_confs_dls_agis,cl_results.preds.dls_agis);
    cl_results.preds.max_confs_dls_scores = cl_results.preds.dls_scores(cl_results.preds.max_confs_dls_local_idxs);
    cl_results.preds.max_confs_dls_n_feats = cl_results.preds.dls_n_feats(cl_results.preds.max_confs_dls_local_idxs);
    cl_results.preds.max_confs_dls_coexps = zeros(1,n_samples);
    for i = 1:n_samples
        cl_results.preds.max_confs_dls_coexps(i) = Coexps(cl_results.preds.max_confs_dls_local_idxs(i),i);
    end
    
    [cl_results.preds.max_scores,cl_results.preds.max_scores_dls_local_idxs] = max(Scores,[],1);
    cl_results.preds.max_scores_dls_agis = cl_results.preds.dls_agis(cl_results.preds.max_scores_dls_local_idxs);
    [temp,cl_results.preds.max_scores_dls_idxs] = ismember(cl_results.preds.max_scores_dls_agis,cl_results.preds.dls_agis);
    
    [cl_results.preds.max_coexps,cl_results.preds.max_coexps_dls_local_idxs] = max(Coexps,[],1);
    cl_results.preds.max_coexps_dls_agis = cl_results.preds.dls_agis(cl_results.preds.max_coexps_dls_local_idxs);
    [temp,cl_results.preds.max_coexps_dls_idxs] = ismember(cl_results.preds.max_coexps_dls_agis,cl_results.preds.dls_agis);
    
    cl_results.preds.scores_sum = sum(Scores,1);
    cl_results.preds.confs_sum = sum(Confs,1);
    cl_results.preds.coexps_sum = sum(abs(Coexps),1);
    
    preds_mask = Confs > conf_th;
    cl_results.preds.n_dls_preds = sum(preds_mask,1);
    Scores(~preds_mask) = 0;
    cl_results.preds.score_sum_preds = sum(Scores,1);
    
    h1_mask = cl_results.preds.n_dls_preds >= min_dls_preds;
    
    H = zeros(size(h1_mask)) - 1;
    H(h1_mask) = 1;
    
    cl_results.preds.H = H';

end