function tr_results = DLSM_train(X,Y,pvals,pval_th,genes_agis,dw,min_coexp,min_feats,w1_feats_th_prctile,min_w1,dls_min_score,nFN,min_FN_coexp,parallel,pot_dls_idxs)
% tr_results = DLSM_train(X,Y,genes_agis,dw,min_feats,dls_min_score,sigma,precW,nFN,parallel);
    
    if dw < 0
        error('dw must be >= 0');
    end
    if min_feats < 2
        error('min_feats must be >= 3');
    end
    
    tr_results.params.pval_th = pval_th;
    tr_results.params.dw = dw;
    tr_results.params.min_coexp = min_coexp;
    tr_results.params.min_feats = min_feats;
    tr_results.params.w1_feats_th_prctile = w1_feats_th_prctile;
    tr_results.params.min_w1 = min_w1;
    tr_results.params.dls_min_score = dls_min_score;
    tr_results.params.nFN = nFN;
    tr_results.params.min_FN_coexp = min_FN_coexp;
    
    opened = false;
    if parallel && matlabpool('size') == 0
        matlabpool('open');
        opened = true;
    end
    
    if ~parallel && matlabpool('size') > 0
        matlabpool close force local
    end
    
    if ~exist('pot_dls_idxs','var')
        pot_dls_idxs = find(Y == 1);
    end
    n1 = numel(pot_dls_idxs);
    dls = cell(n1,1);
    potential_FNs_idxs = [];
    n_valid_dls = 0;
    dls_scores = zeros(n1,1);
    dls_scores_pos = zeros(n1,1);
    dls_idxs = zeros(n1,1);
    dls_n_feats = zeros(n1,1);
    dls_n_avail_feats = zeros(n1,1);
    valid_dls_mask = false(n1,1);
	
	% calculate average number of features in which the genes of the process
	% show a significant response
% 	diff_matrix = zeros(n1,size(X,2));
% 	diff_matrix(pvals(pot_dls_idxs,:)<pval_th) = 1;
% 	w1_feats_th = max(round(prctile(sum(diff_matrix,2),w1_feats_th_prctile)),min_feats+1);
% 	tr_results.params.w1_feats_th = w1_feats_th;
% 	delta_w1 = (1 - min_w1)/(w1_feats_th - min_feats);
    
    % iterate over each class1 sample
    parfor i = 1:n1
        
        dls_idx = pot_dls_idxs(i);
        % train DLS for this sample
        dls{i} = dls_train(X,Y,pvals,pval_th,dls_idx,dw,min_coexp,min_feats,w1_feats_th_prctile,min_w1,dls_min_score,nFN,min_FN_coexp);
%         dls{i} = dls_train_real(X,Y,dls_idx,dw,min_feats,dls_min_score,nFN);
        
        dls_scores(i) = dls{i}.score;
        dls_scores_pos(i) = nan;
        dls_idxs(i) = dls{i}.dls_idx;
        dls_n_feats(i) = numel(dls{i}.feats);
        dls_n_avail_feats(i) = dls{i}.n_avail_feats;
            
        if dls{i}.is_valid
            
            valid_dls_mask(i) = true;
            n_valid_dls = n_valid_dls + 1;
            dls_scores_pos(i) = dls{i}.scores.score_pos;
            
            % get potential FNs genes
            potential_FNs_idxs = union(potential_FNs_idxs,dls{i}.FN_idxs);
            
        end
        
    end
    
    if opened && parallel && matlabpool('size') > 0
        matlabpool close force local
    end
    
    tr_results.X_tr = X;
    tr_results.Y_tr = Y;
    tr_results.genes_agis = genes_agis;
    tr_results.n_dls = n1;
    tr_results.n_valid_dls = n_valid_dls;
    tr_results.dls = dls;
    tr_results.valid_dls_mask = valid_dls_mask;
    tr_results.dls_scores = dls_scores;
    tr_results.dls_scores_pos = dls_scores_pos;
    tr_results.dls_idxs = dls_idxs;
    tr_results.dls_n_feats = dls_n_feats;
    tr_results.dls_n_avail_feats = dls_n_avail_feats;
    tr_results.potential_FNs_idxs = potential_FNs_idxs;
    tr_results.potential_FNs_agis = genes_agis(potential_FNs_idxs);
    tr_results.feats_relevance = zeros(1,size(X,2));
    
    % calculate features relevances
    for i = 1:n1
        if valid_dls_mask(i)
            tr_results.feats_relevance(dls{i}.feats) = tr_results.feats_relevance(dls{i}.feats) + dls{i}.score;
        end
    end

end
