function results = getClass0GenesSet_direct_fast(class1_agis,annot_matrix_all,direct_annot_matrix_all,annot_matrix_nonIEA,max_invalid_terms,min_valid_terms,min_valid_terms_size,max_valid_p1)
% results = getClass0GenesSet_fast(class1_agis,max_p,annot_matrix);
% max_p: discard all genes annotated in terms with a proportion of class1
% genes higher than max_p.

    [tf,class1_idxs] = ismember(class1_agis,annot_matrix_all.rowHeaders);
    
    if ~all(tf)
        not_found_agis = class1_agis(class1_idxs == 0);
        nf_agis_str = '';
        for i = 1:numel(not_found_agis)
            nf_agis_str = [nf_agis_str ', ' not_found_agis{i}];
        end
        error('The following "genes1" AGIs were not found in annot_matrix in "getClass0GenesSet_direct_fast" function: %s',nf_agis_str)
    end
    
    if ~exist('annot_matrix_nonIEA','var')
        annot_matrix_nonIEA = annot_matrix_all;
    end
    
    if nargin < 8
        max_valid_p1 = 0;
    end
    if nargin < 7
        min_valid_terms_size = 2;
    end
    if nargin < 6
        min_valid_terms = 1;
    end
    if nargin < 5
        max_invalid_terms = 0;
    end        
    
    % get the number of genes of class1 annotated in each term
    terms_num_annots1 = sum(annot_matrix_all.matrix(class1_idxs,:),1);
    
    terms_p1 = terms_num_annots1./numel(class1_idxs);
    
    % mark as invalid terms where there are genes of class1 annotated
    invalid_class0_terms_mask = terms_p1 > max_valid_p1;
    
    % get the total number of genes annotated in each term
    terms_num_annots = sum(annot_matrix_all.matrix,1);
    
    % also exclude terms with less than min_valid_terms_size genes annotated
    invalid_class0_terms_mask(terms_num_annots < min_valid_terms_size) = true;
    
    invalid_class0_terms = annot_matrix_all.colHeaders(invalid_class0_terms_mask);
    
    invalid_class0_terms_mask = ismember(direct_annot_matrix_all.colHeaders,invalid_class0_terms);
    valid_class0_terms_mask = ~ismember(annot_matrix_nonIEA.colHeaders,invalid_class0_terms);
    
    % get the final class0 genes as the ones with no direct annotations in
    % the invalid class0 terms, but with at least one direct annotation.
    n_valid_annots = sum(annot_matrix_nonIEA.matrix(:,valid_class0_terms_mask),2);
    n_invalid_annots = sum(direct_annot_matrix_all.matrix(:,invalid_class0_terms_mask),2);
    
    class0_genes_mask = n_valid_annots >= min_valid_terms & n_invalid_annots <= max_invalid_terms;
    
    results.class0_idxs = find(class0_genes_mask);
    results.class0_agis = annot_matrix_all.rowHeaders(class0_genes_mask);

end