function results = get_goterm_sets_direct(go_terms_ids,annot_matrix_all,direct_annot_matrix_all,annot_matrix_nonIEA,max_invalid_terms,min_valid_terms,min_valid_terms_size,max_valid_p1)
    
    go_terms = goid2num(go_terms_ids);
    
    if any(isnan(go_terms))
        error('invalid go_terms! syntaxis: GO:XXXXXXX')
    end
    
    % load default annot_matrices
    if ~exist('annot_matrix_all','var')
        load Arabidopsis_annot_matrix_P_All(2010.09.07).mat
        annot_matrix_all = annot_matrix;
        load Arabidopsis_direct_annot_matrix_P_All(2010.09.07).mat
        direct_annot_matrix_all = direct_annot_matrix;
        load Arabidopsis_annot_matrix_P_nonIEA(2010.09.07).mat
        annot_matrix_nonIEA = annot_matrix;
    end
    
    % check for each term to have annotated genes
    is_error = false;
    for i = 1:numel(go_terms)
        if numel(getClass1GenesSet(go_terms(i),annot_matrix_all)) == 0
            is_error = true;
        end
    end
    if is_error
        results.class1_agis = {};
        results.class0_agis = {};
        results.unlabeled_agis = {};
        return
    end
    
    % get class1 genes
    [class1_agis,class1_idxs] = getClass1GenesSet(go_terms,annot_matrix_all);
    
    results_class0 = getClass0GenesSet_direct_fast(class1_agis,annot_matrix_all,direct_annot_matrix_all,annot_matrix_nonIEA,max_invalid_terms,min_valid_terms,min_valid_terms_size,max_valid_p1);
    
    % get unlabeled genes
    unlabeled_idxs = setdiff(1:numel(annot_matrix_all.rowHeaders),[class1_idxs;results_class0.class0_idxs])';
    unlabeled_agis = annot_matrix_all.rowHeaders(unlabeled_idxs);

    % gather results
    results.class1_agis = class1_agis;
    results.class0_agis = results_class0.class0_agis;
    results.unlabeled_agis = unlabeled_agis;

end