function cv_results = getPrintCVResults(results,conf_th,get_best_conf,precW,dir,name,term_i)
% printResults(filename,results,i);
    addpath ../Evaluation/

    cv_results.params.conf_th = conf_th;
    cv_results.params.get_best_conf = get_best_conf;
    cv_results.params.precW = precW;
    cv_results.cv_params = results.params;

    [n_terms,n_cvreps] = size(results.preds.confs);
    
    % get training labels
    Y = cell(n_terms,1);
    for i = 1:n_terms
        if results.params.use_fns_discovery
            if results.go_terms.discover_FNs
                nclass0 = results.go_terms.nclass0_woFNs(i);
                if results.params.add_FNs
                    nclass1 = results.go_terms.nclass1_wFNs(i);
                end
            else
                error('discover_FNs data not available in results.go_terms')
            end
        else
            nclass0 = results.go_terms.nclass0(i);
            nclass1 = results.go_terms.nclass1(i);
        end
        Y{i} = [ones(nclass1,1); -1*ones(nclass0,1)];
    end
    
    % evaluate predictions
    F = zeros(n_terms,n_cvreps);
    precision = zeros(n_terms,n_cvreps);
    recall = zeros(n_terms,n_cvreps);
    conf_ths = zeros(n_terms,n_cvreps);
    if ~get_best_conf
        conf_ths = conf_ths + conf_th;
    end   
    
    for i = 1:n_terms
        for j = 1:n_cvreps
            if ~isempty(results.preds.confs{i,j})
                if get_best_conf
                    [~,recall_ij,precision_ij,ths_ij] = rocRP(results.preds.confs{i,j},Y{i},false);
                    F_ij = (1+precW^2)./((1./recall_ij) + (precW^2./precision_ij));
                    valid_ths_mask = ths_ij > conf_th;
                    if any(valid_ths_mask)
                        ths_ij = ths_ij(valid_ths_mask);
                        precision_ij = precision_ij(valid_ths_mask);
                        recall_ij = recall_ij(valid_ths_mask);
                        F_ij = F_ij(valid_ths_mask);
                        [F(i,j),max_F_idx] = max(F_ij);
                        conf_ths(i,j) = ths_ij(max_F_idx);
                        precision(i,j) = precision_ij(max_F_idx);
                        recall(i,j) = recall_ij(max_F_idx);
                    else
                        conf_ths(i,j) = conf_th;
                        F(i,j) = NaN;
                        precision(i,j) = NaN;
                        recall(i,j) = 0;
                    end
                else
                    H = results.preds.confs{i,j} > conf_th;
                    [F(i,j),precision(i,j),recall(i,j)] = F_eval(H,Y{i},precW);
                end
            end
        end
    end
    
    cv_results.conf_ths = conf_ths;
    cv_results.F = F;
    cv_results.precision = precision;
    cv_results.recall = recall;

    if get_best_conf
        filename = sprintf('%sCV_GeneGO_BestFConfs_MinConf%.2f_%s_%s.txt',dir,conf_th,name,results.params.test_name);
    else
        filename = sprintf('%sCV_GeneGO_FixedConfTh%.2f_%s_%s.txt',dir,conf_th,name,results.params.test_name);
    end
    fid = fopen(filename,'a');
    
    if exist('term_i','var')
        i = term_i;
        if i == 1
            fprintf(fid,'GO_ID\tGO_NAME\tLEVEL\tMEAN_NDIFF\tNCLASS1\tNCLASS0');
            for j = 1:n_cvreps
                fprintf(fid,'\tF%d',j);
            end
            fprintf(fid,'\tMEAN_F\tSTD_F');
            for j = 1:n_cvreps
                fprintf(fid,'\tPRECISION%d',j);
            end
            fprintf(fid,'\tMEAN_PRECISION\tSTD_PRECISION');
            for j = 1:n_cvreps
                fprintf(fid,'\tRECALL%d',j);
            end
            fprintf(fid,'\tMEAN_RECALL\tSTD_RECALL');
            for j = 1:n_cvreps
                fprintf(fid,'\CONF_TH%d',j);
            end
            fprintf(fid,'\tMEAN_CONF_TH\tSTD_CONF_TH\n');
        end
        fprintf(fid,'%d\t%s\t%d\t%.2f\t%d\t%d', ...
        results.go_terms.id(i),results.go_terms.name{i},results.go_terms.level(i),results.go_terms.mean_numDiff(i), ...
        results.go_terms.nclass1(i),results.go_terms.nclass0(i));
        for j = 1:n_cvreps
            fprintf(fid,'\t%.4f',F(i,j));
        end
        fprintf(fid,'\t%.4f\t%.4f',mean(F(i,:)),std(F(i,:)));
        for j = 1:n_cvreps
            fprintf(fid,'\t%.4f',precision(i,j));
        end
        fprintf(fid,'\t%.4f\t%.4f',mean(precision(i,:)),std(precision(i,:)));
        for j = 1:n_cvreps
            fprintf(fid,'\t%.4f',recall(i,j));
        end
        fprintf(fid,'\t%.4f\t%.4f',mean(recall(i,:)),std(recall(i,:)));
        for j = 1:n_cvreps
            fprintf(fid,'\t%.4f',conf_ths(i,j));
        end
        fprintf(fid,'\t%.4f\t%.4f\n',mean(conf_ths(i,:)),std(conf_ths(i,:)));
    else
        fprintf(fid,'GO_ID\tGO_NAME\tLEVEL\tMEAN_NDIFF\tNCLASS1\tNCLASS0');
        for j = 1:n_cvreps
            fprintf(fid,'\tF%d',j);
        end
        fprintf(fid,'\tMEAN_F\tSTD_F');
        for j = 1:n_cvreps
            fprintf(fid,'\tPRECISION%d',j);
        end
        fprintf(fid,'\tMEAN_PRECISION\tSTD_PRECISION');
        for j = 1:n_cvreps
            fprintf(fid,'\tRECALL%d',j);
        end
        fprintf(fid,'\tMEAN_RECALL\tSTD_RECALL');
        for j = 1:n_cvreps
            fprintf(fid,'\tCONF_TH%d',j);
        end
        fprintf(fid,'\tMEAN_CONF_TH\tSTD_CONF_TH\n');
        
        for i = 1:size(results.F,1)
            fprintf(fid,'%d\t%s\t%d\t%.2f\t%d\t%d', ...
            results.go_terms.id(i),results.go_terms.name{i},results.go_terms.level(i),results.go_terms.mean_numDiff(i), ...
            results.go_terms.nclass1(i),results.go_terms.nclass0(i));
            for j = 1:n_cvreps
                fprintf(fid,'\t%.4f',F(i,j));
            end
            fprintf(fid,'\t%.4f\t%.4f',mean(F(i,:)),std(F(i,:)));
            for j = 1:n_cvreps
                fprintf(fid,'\t%.4f',precision(i,j));
            end
            fprintf(fid,'\t%.4f\t%.4f',mean(precision(i,:)),std(precision(i,:)));
            for j = 1:n_cvreps
                fprintf(fid,'\t%.4f',recall(i,j));
            end
            fprintf(fid,'\t%.4f\t%.4f',mean(recall(i,:)),std(recall(i,:)));
            for j = 1:n_cvreps
                fprintf(fid,'\t%.4f',conf_ths(i,j));
            end
            fprintf(fid,'\t%.4f\t%.4f\n',mean(conf_ths(i,:)),std(conf_ths(i,:)));
        end
    end
    
    fclose(fid);

end
