function results = exprNet_enrich_cv3(matrix,matrix_type,genes_agis,class1_idxs,class0_idxs,corr_cutoff,nfolds,cv_te_idxs)
% results = exprNet_enrich_cv(CorrVector,class1_idxs,class0_idxs,nfolds,precW,corr_cutoffs,cv_tr_idxs,cv_te_idxs);
    
    results.params.corr_cutoff = corr_cutoff;
    results.params.nfolds = nfolds;
    
    tr_genes_idxs = [class1_idxs; class0_idxs];
    Y = [ones(size(class1_idxs)); -ones(size(class0_idxs))];
    
    if ~exist('cv_te_idxs','var')
         [~,cv_te_idxs] = make_crossval_sets(Y,nfolds);
    end
    
    ntotal_genes = numel(genes_agis);
    
    results.tr_genes_idxs = tr_genes_idxs;
    results.cv_te_idxs = cv_te_idxs;
    results.pvals = zeros(size(tr_genes_idxs));
    
    if strcmp(matrix_type,'corrs')
        nbors_mask = getNbors_from_vector_cutoff(CorrVector,tr_genes_idxs,ntotal_genes,corr_cutoff);
    elseif strcmp(matrix_type,'nbors')
        nbors_mask = matrix(tr_genes_idxs,:);
    else
        error('Wrong type of matrix: corrs or nbors')
    end
    
    for i = 1:nfolds
        n_te = numel(cv_te_idxs{i});
        fold_ntotal_genes = ntotal_genes-n_te;
        
        cv_te_total_mask = ismember(1:ntotal_genes,tr_genes_idxs(cv_te_idxs{i}));
        
        num_nbors = sum(nbors_mask(cv_te_idxs{i},~cv_te_total_mask),2);
        
        fold_class1_idxs = setdiff(class1_idxs,tr_genes_idxs(cv_te_idxs{i}));
        fold_n1 = numel(fold_class1_idxs);
        
        num_class1_nbors = sum(nbors_mask(cv_te_idxs{i},fold_class1_idxs),2);
        
        pvals = hygepdf(num_class1_nbors,fold_ntotal_genes,fold_n1,num_nbors);

        p_base = fold_n1/fold_ntotal_genes;
        p_nbors = num_class1_nbors./num_nbors;

        mask = p_nbors <= p_base & pvals < 0.5;
        pvals(mask) = 1 - pvals(mask);
    
        results.pvals(cv_te_idxs{i}) = pvals;
    end

end