function results = exprNet_enrich_predict(matrix,matrix_type,genes_agis,class1_idxs,class0_idxs,corr_cutoff,pval_cutoff,correct_pval)
% results = exprNet_enrich_cv(CorrVector,class1_idxs,class0_idxs,nfolds,precW,corr_cutoffs,cv_tr_idxs,cv_te_idxs);
    
    results.params.corr_cutoff = corr_cutoff;
    results.params.pval_cutoff = pval_cutoff;
    
    ntotal_genes = numel(genes_agis);
    ntr1 = numel(class1_idxs);
    
    tr_genes_idxs = [class1_idxs; class0_idxs];
    results.tr_genes_idxs = tr_genes_idxs;
    
    results.preds.new_genes_idxs = setdiff(1:ntotal_genes,tr_genes_idxs);
    results.preds.new_genes_agis = genes_agis(results.preds.new_genes_idxs);
    
    if strcmp(matrix_type,'corrs')
        nbors_mask = getNbors_from_vector_cutoff(CorrVector,results.preds.new_genes_idxs,ntotal_genes,corr_cutoff);
    elseif strcmp(matrix_type,'nbors')
        nbors_mask = matrix(results.preds.new_genes_idxs,:);
    else
        error('Wrong type of matrix: corrs or nbors')
    end
    
    num_nbors = sum(nbors_mask,2);
    num_class1_nbors = sum(nbors_mask(:,class1_idxs),2);
    pvals = hygepdf(num_class1_nbors,ntotal_genes,ntr1,num_nbors);

    p_base = ntr1/ntotal_genes;
    p_nbors = num_class1_nbors./num_nbors;

    mask = p_nbors <= p_base & pvals < 0.5;
    pvals(mask) = 1 - pvals(mask);
    
    results.preds.pvals = pvals;
    
    if ~exist('pval_cutoff','var')
        results.preds.pval_cutoff = results.tr_pval_cutoff;
    else
        if correct_pval
            results.preds.pval_cutoff = pval_cutoff/numel(pvals);
        else
            results.preds.pval_cutoff = pval_cutoff;
        end
    end
    
    results.preds.pos_idxs = results.preds.new_genes_idxs(pvals < results.preds.pval_cutoff);     
    results.preds.pos_agis = results.preds.new_genes_agis(pvals < results.preds.pval_cutoff);

end