function cv_results = getPrintCVResults(results,pval_th,get_best_pval,correct_pvals,precW,dir,name,print,term_i)
% printResults(filename,results,i);

    cv_results.params.pval_th = pval_th;
    cv_results.params.get_best_pval = get_best_pval;
    cv_results.params.precW = precW;
    cv_results.cv_params = results.params;

    [n_terms,n_cvreps] = size(results.preds.pvals);
    
    if exist('term_i','var')
        i_idxs = term_i;
    else
        i_idxs = 1:n_terms;
    end
    
    % get training labels
    Y = cell(n_terms,1);
    for i = i_idxs
        nclass0 = results.go_terms.nclass0(i);
        nclass1 = results.go_terms.nclass1(i);
        Y{i} = [ones(nclass1,1); -1*ones(nclass0,1)];
    end
    
    % evaluate predictions
    F = zeros(n_terms,n_cvreps);
    precision = zeros(n_terms,n_cvreps);
    recall = zeros(n_terms,n_cvreps);
    pval_ths = zeros(n_terms,n_cvreps);
    if ~get_best_pval
        pval_ths = pval_ths + pval_th;
    end
    
    for i = i_idxs
        for j = 1:n_cvreps
            if ~isempty(results.preds.pvals{i,j})
                if get_best_pval
                    [~,recall_ij,precision_ij,ths_ij] = rocRP(1-results.preds.pvals{i,j},Y{i},false);
                    ths_ij = ths_ij(2:end);
                    ths_ij = 1-ths_ij;
                    precision_ij = precision_ij(1:end-1);
                    recall_ij = recall_ij(1:end-1);
                    F_ij = (1+precW^2)./((1./recall_ij) + (precW^2./precision_ij));
                    valid_ths_mask = ths_ij < pval_th;
                    if any(valid_ths_mask)
                        ths_ij = ths_ij(valid_ths_mask);
                        precision_ij = precision_ij(valid_ths_mask);
                        recall_ij = recall_ij(valid_ths_mask);
                        F_ij = F_ij(valid_ths_mask);
                        [F(i,j),max_F_idx] = max(F_ij);
                        pval_ths(i,j) = ths_ij(max_F_idx);
                        precision(i,j) = precision_ij(max_F_idx);
                        recall(i,j) = recall_ij(max_F_idx);
                    else
                        pval_ths(i,j) = pval_th;
                        F(i,j) = NaN;
                        precision(i,j) = NaN;
                        recall(i,j) = 0;
                    end
                else
                    if correct_pvals
                        pval_ths(i,j) = pval_th/numel(results.preds.pvals{i,j});
                    end
                    H = results.preds.pvals{i,j} < pval_ths(i,j);
                    [F(i,j),precision(i,j),recall(i,j)] = F_eval(H,Y{i},precW);
                end
            end
        end
    end
    
    cv_results.pval_ths = pval_ths;
    cv_results.F = F;
    cv_results.precision = precision;
    cv_results.recall = recall;

    if print
        filename = sprintf('%sResults_CV_%s.txt',dir,name);
        fid = fopen(filename,'a');

        if exist('term_i','var')
            i = term_i;
            if i == 1
                fprintf(fid,'GO_ID\tGO_NAME\tLEVEL\tMEAN_NDIFF\tNCLASS1\tNCLASS0');
                for j = 1:n_cvreps
                    fprintf(fid,'\tF%d',j);
                end
                fprintf(fid,'\tMEAN_F\tSTD_F');
                for j = 1:n_cvreps
                    fprintf(fid,'\tPRECISION%d',j);
                end
                fprintf(fid,'\tMEAN_PRECISION\tSTD_PRECISION');
                for j = 1:n_cvreps
                    fprintf(fid,'\tRECALL%d',j);
                end
                fprintf(fid,'\tMEAN_RECALL\tSTD_RECALL');
                for j = 1:n_cvreps
                    fprintf(fid,'\tPVAL_TH%d',j);
                end
                fprintf(fid,'\tMEAN_PVAL_TH\tSTD_PVAL_TH\n');
            end
            fprintf(fid,'%d\t%s\t%d\t%.2f\t%d\t%d', ...
            results.go_terms.id(i),results.go_terms.name{i},results.go_terms.level(i),results.go_terms.mean_numDiff(i), ...
            results.go_terms.nclass1(i),results.go_terms.nclass0(i));
            for j = 1:n_cvreps
                fprintf(fid,'\t%.4f',F(i,j));
            end
            fprintf(fid,'\t%.4f\t%.4f',mean(F(i,:)),std(F(i,:)));
            for j = 1:n_cvreps
                fprintf(fid,'\t%.4f',precision(i,j));
            end
            fprintf(fid,'\t%.4f\t%.4f',mean(precision(i,:)),std(precision(i,:)));
            for j = 1:n_cvreps
                fprintf(fid,'\t%.4f',recall(i,j));
            end
            fprintf(fid,'\t%.4f\t%.4f',mean(recall(i,:)),std(recall(i,:)));
            for j = 1:n_cvreps
                fprintf(fid,'\t%.2e',pval_ths(i,j));
            end
            fprintf(fid,'\t%.2e\t%.2e\n',mean(pval_ths(i,:)),std(pval_ths(i,:)));
        else
            fprintf(fid,'GO_ID\tGO_NAME\tLEVEL\tMEAN_NDIFF\tNCLASS1\tNCLASS0');
            for j = 1:n_cvreps
                fprintf(fid,'\tF%d',j);
            end
            fprintf(fid,'\tMEAN_F\tSTD_F');
            for j = 1:n_cvreps
                fprintf(fid,'\tPRECISION%d',j);
            end
            fprintf(fid,'\tMEAN_PRECISION\tSTD_PRECISION');
            for j = 1:n_cvreps
                fprintf(fid,'\tRECALL%d',j);
            end
            fprintf(fid,'\tMEAN_RECALL\tSTD_RECALL');
            for j = 1:n_cvreps
                fprintf(fid,'\tPVAL_TH%d',j);
            end
            fprintf(fid,'\tMEAN_PVAL_TH\tSTD_PVAL_TH\n');

            for i = i_idxs
                fprintf(fid,'%d\t%s\t%d\t%.2f\t%d\t%d', ...
                results.go_terms.id(i),results.go_terms.name{i},results.go_terms.level(i),results.go_terms.mean_numDiff(i), ...
                results.go_terms.nclass1(i),results.go_terms.nclass0(i));
                for j = 1:n_cvreps
                    fprintf(fid,'\t%.4f',F(i,j));
                end
                fprintf(fid,'\t%.4f\t%.4f',mean(F(i,:)),std(F(i,:)));
                for j = 1:n_cvreps
                    fprintf(fid,'\t%.4f',precision(i,j));
                end
                fprintf(fid,'\t%.4f\t%.4f',mean(precision(i,:)),std(precision(i,:)));
                for j = 1:n_cvreps
                    fprintf(fid,'\t%.4f',recall(i,j));
                end
                fprintf(fid,'\t%.4f\t%.4f',mean(recall(i,:)),std(recall(i,:)));
                for j = 1:n_cvreps
                    fprintf(fid,'\t%.2e',pval_ths(i,j));
                end
                fprintf(fid,'\t%.2e\t%.2e\n',mean(pval_ths(i,:)),std(pval_ths(i,:)));
            end
        end

        fclose(fid);
        fprintf('\nCV results details saved in:\n%s\n', filename)
    end

end
