% Script to run libSVM

% `svm-train' Options
% =================
% -s svm_type : set type of SVM (default 0)
% 	0 -- C-SVC
% 	1 -- nu-SVC
% 	2 -- one-class SVM
% 	3 -- epsilon-SVR
% 	4 -- nu-SVR
% -t kernel_type : set type of kernel function (default 2)
% 	0 -- linear: u'*v
% 	1 -- polynomial: (gamma*u'*v + coef0)^degree
% 	2 -- radial basis function: exp(-gamma*|u-v|^2)
% 	3 -- sigmoid: tanh(gamma*u'*v + coef0)
% 	4 -- precomputed kernel (kernel values in training_set_file)
% -d degree : set degree in kernel function (default 3)
% -g gamma : set gamma in kernel function (default 1/num_features)
% -r coef0 : set coef0 in kernel function (default 0)
% -c cost : set the parameter C of C-SVC, epsilon-SVR, and nu-SVR (default 1)
% -n nu : set the parameter nu of nu-SVC, one-class SVM, and nu-SVR (default 0.5)
% -p epsilon : set the epsilon in loss function of epsilon-SVR (default 0.1)
% -m cachesize : set cache memory size in MB (default 100)
% -e epsilon : set tolerance of termination criterion (default 0.001)
% -h shrinking : whether to use the shrinking heuristics, 0 or 1 (default 1)
% -b probability_estimates : whether to train a SVC or SVR model for probability estimates, 0 or 1 (default 0)
% -wi weight : set the parameter C of class i to weight*C, for C-SVC (default 1)
% -v n: n-fold cross validation mode
% -q : quiet mode (no outputs)
% 
% 
% The k in the -g option means the number of attributes in the input data.
% 
% option -v randomly splits the data into n parts and calculates cross
% validation accuracy/mean squared error on them.

% `svm-predict' Options
% ===================
% -b probability_estimates: whether to predict probability estimates, 0 or 1 (default 0); for one-class SVM only 0 is supported

% Input Data:
% Y_tr: Vector with the labels of each training sample.
% X_tr: Training data matrix with samples in rows and features in columns.
% Y_te: Vector with the labels of each test sample. To classify new samples
%
% (Y_te unknown) you can use any vector as input (e.g. an all zero vector).
% X_tr: Training data matrix with samples in rows and features in columns.

% Outputs:
% model: the trained svm model
% H_te: the classification (labels) given to each test sample. (No estoy
% seguro si funciona con cualquier tipo de label... to uso 1 y -1)
% Acc_te: the accuracy of the classificacion (based in the Y_te provided).
% Prob_te: the probability of each sample (rows) to belong to each class
% (columns). For two classes (1,-1), column 1 corresponds to the positive
% class and column 2 to negative class probabilities.

tr_params = '-s 0 -t 1 -d 2 -r 1 -b 1';
model = svmtrain(Y_tr, X_tr, tr_params);

te_params =  '-b 1';
[H_te, Acc_te, Prob_te] = svmpredict(Y_te, X_te, model, te_params);



