% Script to run Coexpression Networks (CN) cross-validation test over the 
% go-terms defined by Paper_test_1_Representative_GO_terms_script.m

%% addpaths

addpath ../Other_methods/SVM/
addpath ../Other_methods/SVM/libsvm-mat-3.0-1/
addpath ../Functions/GO_terms/
addpath ../Functions/Evaluation/
addpath ../Data/

%% define parameters
test_name = 'SVM_RBF'; results.params.test_name = test_name;
directory = '../Results/Expert/';
if ~exist(directory,'dir')
    mkdir(directory);
end

terms_data_dir = '../Data/';
name = 'Bioinf_paper';
year = 2008;
min_annots = 30;
max_annots = 500;
cv_nfolds = 10;
n_cvreps = 3;

precW = 2;
conf_th = 0.5;
parallel = true;

% -s 0: C-SVM
% -t 2: radial basis kernel: exp(-gamma*|u-v|^2)
% -b 1: probability_estimates
tr_params = '-s 0 -t 2 -b 1 w+1 2 w-1 1'; % Radial Basis
te_params = '-b 1';

%% load data

tic
disp('Loading data...')

% load expression data
load ExpData_expert.mat

% load go-terms data
load(sprintf('GO_Terms_Data_year%d_min%d_max%d_cvnfolds%d_%s.mat',year,min_annots,max_annots,cv_nfolds,name))
results.go_terms = go_terms;
clear go_terms
toc

%% Run crossval

disp('Starting cross-validation analisys...')
i_start = 66;

n_terms = numel(results.go_terms.id);

if i_start == 1
    results.F = zeros(n_terms,n_cvreps);
    results.precision = zeros(n_terms,n_cvreps);
    results.recall = zeros(n_terms,n_cvreps);
    results.preds.confs = cell(n_terms,n_cvreps);
    results.preds.fps_agis = cell(n_terms,n_cvreps);
    results.preds.tps_agis = cell(n_terms,n_cvreps);
end

if parallel && matlabpool('size') == 0
        matlabpool open
        opened = true;
end

if ~parallel && matlabpool('size') > 0
        matlabpool close
end

t1 = tic;


for i = i_start:n_terms
    
    t2 = tic;
    
    for j = 1:n_cvreps
        fprintf('\nGO:%d, name: %s\nnclass1: %d, nclass0: %d\nmean_numDiff: %.2f\nReplicate: %d\n',results.go_terms.id(i),results.go_terms.name{i},results.go_terms.nclass1(i),results.go_terms.nclass0(i),results.go_terms.mean_numDiff(i),j);
        
        class1_idxs = results.go_terms.class1_idxs{i};
        class1_agis = results.go_terms.class1_agis{i};
        class0_idxs = results.go_terms.class0_idxs{i};
        class0_agis = results.go_terms.class0_agis{i};
        nclass0 = results.go_terms.nclass0(i);
        tr_idxs = results.go_terms.tr_idxs{i,j};
        te_idxs = results.go_terms.te_idxs{i,j};

        [X,Y,genes_agis_X] = makeXY(ExpData.LogRatio,class1_agis,class0_agis,ExpData.genes_agis);
        term_results = libSVM_CV(X,Y,numel(tr_idxs),tr_params,te_params,conf_th,precW,tr_idxs,te_idxs);

        results.F(i,j) = term_results.F;
        results.precision(i,j) = term_results.precision;
        results.recall(i,j) = term_results.recall;
        results.conf_th(i,j) = conf_th;
        genes_agis = [results.go_terms.class1_agis{i}; class0_agis];
        results.preds.confs{i,j} = term_results.Confs;
        results.preds.fps_agis{i,j} = genes_agis(term_results.H == 1 & Y == -1);
        results.preds.tps_agis{i,j} = genes_agis(term_results.H == 1 & Y == 1);
        
        fprintf('\nRep%d Done! Conf_th:%.1f, F:%.4f, Precision:%.3f, Recall:%.3f\n',j,conf_th,results.F(i,j),results.precision(i,j),results.recall(i,j))
    
        save(sprintf('%sResults_CV_%s.mat',directory,test_name),'results')
        
        toc(t2);
    end
    
    printCVResults(sprintf('%sResults_CV_%s.txt',directory,test_name),results,i);
    
end

if matlabpool('size') > 0
    matlabpool close
end
toc(t1);

fprintf('\nReady!! Results saved in:\n%s\n',sprintf('%sResults_CV_%s.txt',directory,test_name))

