% Script to run DLS enrichment test over the go-terms defined by
% Get_GOTerms_Data_Script.m

%% add paths

addpath ../Functions/DLS_Core/
addpath ../Functions/GO_terms/
addpath ../Functions/Evaluation/
addpath ../Functions/PrintsResults/
addpath ../Data

%% params
test_name = 'Automated2000_DLS'; results.params.test_name = test_name;
directory = '../Results/Automated/';
if ~exist(directory,'dir')
    mkdir(directory);
end

terms_data_dir = '../Data/';
name = 'Bioinf_paper';
year = 2008;
min_annots = 30;
max_annots = 500;
cv_nfolds = 10;

FDR_th = 0.1; results.params.FDR_th = FDR_th;
dw = 3; results.params.dw = dw;
pFN = 0.05; results.params.pFN = pFN;
conf_th = 0.5; results.params.conf_th = conf_th;
min_feats = 5; results.params.min_feats = min_feats;
w1_feats_th_prctile = 30; results.params.w1_feats_th_prctile = w1_feats_th_prctile;
min_w1 = 0.6; results.params.min_w1 = min_w1;
sigma = 0.1; results.params.sigma = sigma;
min_coexp = 0.6; results.params.min_coexp = min_coexp;

min_FN_coexp = 0.4; results.params.min_FN_coexp = min_FN_coexp;
sign_min_score = 0; results.params.sign_min_score = sign_min_score;
precW = 2; results.params.precW = precW;
p_dls_preds = 0; results.params.p_dls_preds = p_dls_preds;

use_fns_discovery = false; results.params.use_fns_discovery = use_fns_discovery;
add_FNs = true; results.params.add_FNs = add_FNs;

parallel = true;
    
%% load data

% load expression data
disp('Loading data...')
load ExpData_automated.mat

ExpData.LogRatio = ExpData.LogRatio(:,ExpData.feats2000_idxs);
ExpData.FDR = ExpData.FDR(:,ExpData.feats2000_idxs);

% load go-terms data
if ~isfield(results,'go_terms')
    load(sprintf('GO_Terms_Data_year%d_min%d_max%d_cvnfolds%d_%s.mat',year,min_annots,max_annots,cv_nfolds,name))
    results.go_terms = go_terms;
    clear go_terms
end

%% Run DLS method

t = tic;
n_terms = numel(results.go_terms.id);
results.conf_th = zeros(n_terms,1) + conf_th;
disp('Training and making predictions using year 2008 annotations...')

if ~isfield(results,'preds')
    results.preds.tr_agis = cell(n_terms,1);
    results.preds.tr_confs = cell(n_terms,1);
    results.preds.new_agis = cell(n_terms,1);
    results.preds.new_confs = cell(n_terms,1);
    results.preds.fns_agis = cell(n_terms,1);
    results.preds.fns_confs = cell(n_terms,1);
    results.preds.n_pos_new = zeros(n_terms,1)-1;
    results.preds.n_pos_fns = zeros(n_terms,1)-1;
    results.preds.tr_ntps = zeros(n_terms,1)-1;
    results.preds.tr_nfps = zeros(n_terms,1)-1;
    results.preds.tr_precision = zeros(n_terms,1)-1;
    results.preds.tr_recall = zeros(n_terms,1)-1;
    results.preds.tr_F = zeros(n_terms,1)-1;
    if ~isfield(results.go_terms,'discover_FNs')
        results.go_terms.discover_FNs = false;
    end
end

i_start = 1;
i_end = n_terms;


for i = i_start:3:i_end
    
    tic
    
    if use_fns_discovery
        if results.go_terms.discover_FNs
            class0_agis = results.go_terms.class0_agis_woFNs{i};
            nclass0 = results.go_terms.nclass0_woFNs(i);
            if add_FNs
                class1_agis = results.go_terms.class1_agis_wFNs{i};
                nclass1 = results.go_terms.nclass1_wFNs(i);
            else
                class1_agis = results.go_terms.class1_agis{i};
                nclass1 = results.go_terms.nclass1(i);
            end
        else
            error('discover_FNs data not available in results.go_terms')
        end
    else
        class0_agis = results.go_terms.class0_agis{i};
        nclass0 = results.go_terms.nclass0(i);
        class1_agis = results.go_terms.class1_agis{i};
        nclass1 = results.go_terms.nclass1(i);
    end
    
    fprintf('\nGO:%d, name: %s\nnclass1: %d, nclass0: %d\nmean_numDiff: %.2f\n',results.go_terms.id(i),results.go_terms.name{i},nclass1,nclass0,results.go_terms.mean_numDiff(i));
    
    if results.preds.n_pos_new(i) < 0
    
        disp('Training...')
        [X,Y,genes_agis_X,pvals_X] = makeXY(ExpData.LogRatio,class1_agis,class0_agis,ExpData.genes_agis,ExpData.FDR);

        nFN = ceil(numel(class0_agis)*pFN/100);
        TR_results = DLSM_train(X,Y,pvals_X,FDR_th,genes_agis_X,dw,min_coexp,min_feats,w1_feats_th_prctile,min_w1,sign_min_score,nFN,min_FN_coexp,parallel);
        disp('Training ready!')
        min_dls_preds = ceil(p_dls_preds*TR_results.n_valid_dls);
        fprintf('min_dls_preds: %d\n',min_dls_preds);

        conf_th = results.conf_th(i);

        fprintf('Classifying new genes with conf_th = %.2f\n',results.conf_th(i))
        [unlabeled_agis,unlabeled_idxs] = setdiff(ExpData.genes_agis,union(class1_agis,class0_agis));
        X_unlabeled = ExpData.LogRatio(unlabeled_idxs,:);

        new_results = DLSM_classify(X_unlabeled,unlabeled_agis,TR_results,sigma,parallel,conf_th,min_coexp,min_dls_preds,false);
        results.preds.n_pos_new(i) = numel(find(new_results.preds.H == 1));
        results.preds.new_agis{i} = unlabeled_agis;
        results.preds.new_confs{i} = new_results.preds.max_confs;
        
    end
    
    fprintf('Num new positive genes: %d\n',results.preds.n_pos_new(i));
    
    disp('Saving...')
    save(sprintf('%sResults_Enrichment_%s.mat',directory,test_name),'results')
    
    toc
    
end

save(sprintf('%sResults_Enrichment_%s.mat',directory,test_name),'results')
disp('Ready!!')
toc(t);

%% Enrichment analysis

t = tic;
disp('Loading year 2010 annotations...')
load Arabidopsis_annot_matrix_P_nonIEA(2010.09.07).mat

filename = strcat(directory,'Results_Enrichment_2008_2010_',results.params.test_name,'.txt');

enrichment_analysis_DLS(results,annot_matrix,filename);
toc(t)

fprintf('\n\nAll done! Enrichmente analysis results saved at:\n%s\n',filename);



