% Script to run DLS enrichment test over the go-terms defined by
% Get_GOTerms_Data_Script.m

%% add paths

addpath ../Other_methods/CN/
addpath ../Functions/GO_terms/
addpath ../Functions/Evaluation/
addpath ../Data/

%% params
test_name = 'Automated3911_CN'; results.params.test_name = test_name;
directory = '../Results/Automated/';
if ~exist(directory,'dir')
    mkdir(directory);
end

terms_data_dir = '../Data/';
name = 'Bioinf_paper';
year = 2008;
min_annots = 30;
max_annots = 500;
cv_nfolds = 10;

corr_cutoffs = 0.6; results.params.corr_cutoffs = corr_cutoffs;
precW = 2; results.params.precW = precW;

pval_th = 0.1; results.params.pval_th = pval_th;
correct_pval = true; results.params.correct_pval = correct_pval;
    
%% load data

% load pre-computed correlations
load ExpData_automated.mat

% load go-terms data
load(sprintf('GO_Terms_Data_year%d_min%d_max%d_cvnfolds%d_%s.mat',year,min_annots,max_annots,cv_nfolds,name))
results.go_terms = go_terms;
clear go_terms

%% Construct Network

if ~exist('CorrVector','var') && ~exist('nborsMatrix','var')
    CorrVector = getCorrVector(ExpData.LogRatio,'cosine');
    clear ExpData
    disp('Ready')
end

% annotations data
load Arabidopsis_annot_matrix_P_All(2008.05.08).mat
genes_agis = annot_matrix.rowHeaders;
ntotal_genes = numel(genes_agis);
clear annot_matrix

if ~exist('nborsMatrix','var')
    disp('Constructing network...')
    nborsMatrix = getNbors_from_vector_cutoff(CorrVector,1:ntotal_genes,ntotal_genes,corr_cutoffs);
    clear CorrVector
end

%% Run method

t = tic;
n_terms = numel(results.go_terms.id);
results.pval_th = zeros(n_terms,1) + pval_th;

if isfield(results,'preds')
    i_start = find(cellfun(@isempty,results.preds.new_agis),1,'first');
else
    i_start = 1;
    results.preds.new_agis = cell(n_terms,1);
    results.preds.new_pvals = cell(n_terms,1);
    results.preds.n_pos_new = zeros(n_terms,1);
    results.preds.tr_F = zeros(n_terms,1);
end

for i = i_start:n_terms
    
    tic
    
    fprintf('\nGO:%d, name: %s\nnclass1: %d, nclass0: %d\nmean_numDiff: %.2f\n',results.go_terms.id(i),results.go_terms.name{i},results.go_terms.nclass1(i),results.go_terms.nclass0(i),results.go_terms.mean_numDiff(i));
    
    disp('Making predictions...')
    preds_results = exprNet_enrich_predict(nborsMatrix,'nbors',genes_agis,results.go_terms.class1_idxs{i},results.go_terms.class0_idxs{i},corr_cutoffs,results.pval_th(i),correct_pval);
    
    results.preds.new_agis{i} = preds_results.preds.new_genes_agis;
    results.preds.new_pvals{i} = preds_results.preds.pvals;
    results.preds.n_pos_new(i) = numel(preds_results.preds.pos_idxs);
    results.pval_th(i) = preds_results.preds.pval_cutoff;
    
    fprintf('Num new positive preds: %d\n',results.preds.n_pos_new(i))
    
    toc
    
end

save_name = sprintf('%sResults_Preds_%s.mat',directory,test_name);
save(save_name,'results')
disp('Ready!!')
toc(t);

%% Enrichment analysis

t = tic;
disp('Loading year 2010 annotations...')
load Arabidopsis_annot_matrix_P_nonIEA(2010.09.07).mat

filename = strcat(directory,'Enrichment2008_2010_results_',results.params.test_name,'.txt');

enrichment_analysis_CN(results,annot_matrix,filename);
toc(t)

fprintf('\n\nAll done! Enrichmente analysis results saved at:\n%s\n',filename);



