% Script to run DLS enrichment test over the go-terms defined by
% Get_GOTerms_Data_Script.m

%% add paths

addpath ../Other_methods/SVM/
addpath ../Other_methods/SVM/libsvm-mat-3.0-1/
addpath ../Functions/GO_terms/
addpath ../Functions/Evaluation/
addpath ../Data/

%% params
test_name = 'Automated2000_SMV_RBF'; results.params.test_name = test_name;
directory = '../Results/Automated/SVM_Params/';
if ~exist(directory,'dir')
    mkdir(directory);
end

terms_data_dir = '../Data/';
name = 'Bioinf_paper';
year = 2008;
min_annots = 30;
max_annots = 500;
cv_nfolds = 10;

precW = 2; results.params.precW = precW;

% -s 0: C-SVM
% -t 2: radial basis kernel: exp(-gamma*|u-v|^2)1
% -b 1: probability_estimates
tr_params = '-s 0 -t 2 -b 1 w+1 2 w-1 1'; results.params.tr_params = tr_params; %rbf
te_params = '-b 1'; results.params.te_params = te_params;

conf_th = 0.5; results.params.conf_th = conf_th;
parallel = true;
    
%% load data

% load expression data
load ExpData_automated.mat

ExpData.LogRatio = ExpData.LogRatio(:,ExpData.feats2000_idxs);
ExpData.FDR = ExpData.FDR(:,ExpData.feats2000_idxs);

% load go-terms data
load(sprintf('GO_Terms_Data_year%d_min%d_max%d_cvnfolds%d_%s.mat',year,min_annots,max_annots,cv_nfolds,name))
results.go_terms = go_terms;
clear go_terms

%% Run method

t = tic;
n_terms = numel(results.go_terms.id);
results.conf_th = zeros(n_terms,1) + conf_th;

if isfield(results,'preds')
    i_start = find(cellfun(@isempty,results.preds.new_agis),1,'first');
else
    i_start = 1;
    results.preds.new_agis = cell(n_terms,1);
    results.preds.new_confs = cell(n_terms,1);
    results.preds.n_pos_new = zeros(n_terms,1);
    results.preds.tr_F = zeros(n_terms,1);
    results.preds.tr_precision = zeros(n_terms,1);
    results.preds.tr_recall = zeros(n_terms,1);
end


for i = i_start:n_terms
    
    tic
    
    fprintf('\nGO:%d, name: %s\nnclass1: %d, nclass0: %d\nmean_numDiff: %.2f\n',results.go_terms.id(i),results.go_terms.name{i},results.go_terms.nclass1(i),results.go_terms.nclass0(i),results.go_terms.mean_numDiff(i));
    
    class1_agis = results.go_terms.class1_agis{i};
    class0_agis = results.go_terms.class0_agis{i};
    nclass0 = results.go_terms.nclass0(i);
    
    disp('Training...')
    [X,Y,genes_agis_X] = makeXY(ExpData.LogRatio,class1_agis,class0_agis,ExpData.genes_agis);
    
    [~,gamma] = getParamsSVM(X, Y);
    tr_params2 = sprintf('%s -g %.4f',tr_params,gamma);
    model = svmtrain(Y, X, tr_params2);
    
    fprintf('Classifying training set with conf_th = %.2f\n',results.conf_th(i))
    [~, ~, Confs] = svmpredict(Y, X, model, te_params);
    Confs = Confs(:,1);
    H = Confs > results.conf_th(i);
    [results.preds.tr_F(i),results.preds.tr_precision(i),results.preds.tr_recall(i)] = F_eval(H,Y,precW);
    fprintf('F: %.2f, Precision: %.2f, Recall: %.2f\n',results.preds.tr_F(i),results.preds.tr_precision(i),results.preds.tr_recall(i))

    disp('classifying new genes...')
    [unlabeled_agis,unlabeled_idxs] = setdiff(ExpData.genes_agis,union(results.go_terms.class1_agis{i},results.go_terms.class0_agis{i}));
    X_unlabeled = ExpData.LogRatio(unlabeled_idxs,:);
    [~, ~, Confs] = svmpredict(zeros(numel(unlabeled_idxs),1), X_unlabeled, model, te_params);
    Confs = Confs(:,1);
    H = Confs > results.conf_th(i);
    
    results.preds.new_agis{i} = unlabeled_agis;
    results.preds.new_confs{i} = Confs;
    results.preds.n_pos_new(i) = numel(find(H));
    fprintf('Num new positive genes: %d\n',results.preds.n_pos_new(i));
        
    printMakePredsResults(sprintf('%sResults_Preds_%s.mat',directory,test_name),results,i);
    
    toc
    
end

save_name = sprintf('%sResults_Preds_SVM_%s.mat',directory,test_name);
save(save_name,'results')
disp('Ready!!')
toc(t);

%% Enrichment analysis

t = tic;
disp('Loading year 2010 annotations...')
load Arabidopsis_annot_matrix_P_nonIEA(2010.09.07).mat

filename = strcat(directory,'Results_Enrichment_2008_2010_',results.params.test_name,'.txt');

enrichment_analysis_SVM(results,annot_matrix,filename);
toc(t)

fprintf('\n\nAll done! Enrichmente analysis results saved at:\n%s\n',filename);
