% Script to apply the False Negative Discovery (FND) method to the go-terms 
% defined using Representative_GO_terms_script.m

%% addpaths

addpath ../Functions/DLS_Core/
addpath ../Functions/GO_terms/
addpath ../Functions/Evaluation/
addpath ../Data

%% params
directory = '../Data/';
if ~exist(directory,'dir')
    mkdir(directory);
end

terms_data_name = 'Bioinf_paper';
year = 2008;
min_annots = 30;
max_annots = 500;
cv_nfolds = 10;
n_cvreps = 3;

FDR_th = 0.1;
dw = 9;
pFN = 0.5;
min_feats = 5;
w1_feats_th_prctile = 40;
min_w1 = 0.6;
sigma = 0.1;
min_coexp = 0.6;

conf_th = 0.5;
dls_min_score = 0;
p_dls_preds = 0;

add_FNs = true;
fns_pFN = 0.5;
min_FN_coexp = 0.4;
score_pos_prctile = 50;

max_iters = 10;
max_p_fns_per_iter = 0.02;

parallel = true;

%% load data

% load expression data
load ExpData_expert.mat

% load go-terms data
if ~exist('go_terms','var')
    load(sprintf('GO_Terms_Data_year%d_min%d_max%d_cvnfolds%d_%s.mat',year,min_annots,max_annots,cv_nfolds,terms_data_name));

    go_terms.fns_params.add_FNs = add_FNs;
    go_terms.fns_params.FDR_th = FDR_th;
    go_terms.fns_params.min_feats = min_feats;
    go_terms.fns_params.w1_feats_th_prctile = w1_feats_th_prctile;
    go_terms.fns_params.min_w1 = min_w1;
    go_terms.fns_params.pFN = pFN;
    go_terms.fns_params.fns_pFN = fns_pFN;
    go_terms.fns_params.min_FN_coexp = min_FN_coexp;
    go_terms.fns_params.score_pos_prctile = score_pos_prctile;
    go_terms.fns_params.max_iters = max_iters;
    go_terms.fns_params.max_p_fns_per_iter = max_p_fns_per_iter;
    go_terms.fns_params.dw = dw;
    go_terms.fns_params.dls_min_score = dls_min_score;
    go_terms.fns_params.sigma = sigma;
    go_terms.fns_params.conf_th = conf_th;
    go_terms.fns_params.min_coexp = min_coexp;
    go_terms.fns_params.p_dls_preds = p_dls_preds;
end

%% Run method

t = tic;

save_name = sprintf('%sGO_Terms_Data_year%d_min%d_max%d_cvnfolds%d_%s.mat',directory,year,min_annots,max_annots,cv_nfolds,terms_data_name);
n_terms = numel(go_terms.id);

if isfield(go_terms,'tr_idxs_woFNs')
    i_start = find(cellfun(@isempty,go_terms.tr_idxs_woFNs),1,'first');
else
    i_start = 1;
    go_terms.discover_FNs = true;
    go_terms.fns_agis = cell(n_terms,1);
    go_terms.fns_idxs = cell(n_terms,1);
    go_terms.fns_agis_confs = cell(n_terms,1);
    go_terms.class0_agis_woFNs = cell(n_terms,1);
    go_terms.class0_idxs_woFNs = cell(n_terms,1);
    go_terms.class1_agis_wFNs = cell(n_terms,1);
    go_terms.class1_idxs_wFNs = cell(n_terms,1);
    go_terms.nclass0_woFNs = zeros(n_terms,1);
    go_terms.nclass1_wFNs = zeros(n_terms,1);
    go_terms.tr_idxs_woFNs = cell(n_terms,n_cvreps);
    go_terms.te_idxs_woFNs = cell(n_terms,n_cvreps);
    go_terms.tr_idxs_woFNs_addFNs = cell(n_terms,n_cvreps);
    go_terms.te_idxs_woFNs_addFNs = cell(n_terms,n_cvreps);
end

for i = i_start:n_terms
    
    tic
    
    eval_terms_results = evalTermsDiffExp(ExpData.DiffExp_FDR01,ExpData.genes_agis,go_terms.class1_agis(i),30);
    go_terms.mean_numDiff(i) = eval_terms_results.mean_numDiff;
    
    fprintf('\nGO:%d, name: %s\nnclass1: %d, nclass0: %d\nAvg DiffExp: %.2f\n',go_terms.id(i),go_terms.name{i},go_terms.nclass1(i),go_terms.nclass0(i),go_terms.mean_numDiff(i));
    
    
    disp('Discovering potential false negatives...')

    FNs_results = FND(ExpData,ExpData.FDR,FDR_th,go_terms.class1_agis{i},go_terms.class0_agis{i},conf_th,p_dls_preds,max_iters,max_p_fns_per_iter,dw,dw,min_feats,w1_feats_th_prctile,min_w1,dls_min_score,sigma,min_coexp,fns_pFN,pFN,min_FN_coexp,score_pos_prctile,add_FNs,parallel);

    % extract FNs results
    if ~isempty(FNs_results.valid_FNs_agis)
        go_terms.fns_agis{i} = FNs_results.valid_FNs_agis;
        
        [~,go_terms.fns_idxs{i}] = ismember(go_terms.fns_agis{i},ExpData.genes_agis);

        go_terms.class0_agis_woFNs{i} = setdiff(go_terms.class0_agis{i},go_terms.fns_agis{i});
        go_terms.class0_idxs_woFNs{i} = setdiff(go_terms.class0_idxs{i},go_terms.fns_idxs{i});
        
        go_terms.nclass0_woFNs(i) = numel(go_terms.class0_agis_woFNs{i});
        
        go_terms.class1_agis_wFNs{i} = union(go_terms.class1_agis{i},go_terms.fns_agis{i});
        go_terms.class1_idxs_wFNs{i} = union(go_terms.class1_idxs{i},go_terms.fns_idxs{i});

        go_terms.nclass1_wFNs(i) = numel(go_terms.class1_agis_wFNs{i});

        % get new cross-validation sets
        Y = [ones(go_terms.nclass1(i),1); -ones(numel(go_terms.class0_idxs_woFNs{i}),1)];
        for j = 1:n_cvreps
            [go_terms.tr_idxs_woFNs{i,j},go_terms.te_idxs_woFNs{i,j}] = make_crossval_sets(Y,cv_nfolds);
        end
        if add_FNs
            Y = [ones(go_terms.nclass1_wFNs(i),1); -ones(numel(go_terms.class0_idxs_woFNs{i}),1)];
            for j = 1:n_cvreps
                [go_terms.tr_idxs_woFNs_addFNs{i,j},go_terms.te_idxs_woFNs_addFNs{i,j}] = make_crossval_sets(Y,cv_nfolds);
            end
        end
    else
        go_terms.class0_agis_woFNs{i} = go_terms.class0_agis{i};
        go_terms.class0_idxs_woFNs{i} = go_terms.class0_idxs{i};

        go_terms.nclass0_woFNs(i) = numel(go_terms.class0_agis_woFNs{i});
        
        go_terms.class1_agis_wFNs{i} = go_terms.class1_agis{i};
        go_terms.class1_idxs_wFNs{i} = go_terms.class1_idxs{i};

        go_terms.nclass1_wFNs(i) = numel(go_terms.class1_agis_wFNs{i});

        % get new cross-validation sets
        go_terms.tr_idxs_woFNs(i,:) = go_terms.tr_idxs(i,:);
        go_terms.te_idxs_woFNs(i,:) = go_terms.te_idxs(i,:);
        
        go_terms.tr_idxs_woFNs_addFNs(i,:) = go_terms.tr_idxs(i,:);
        go_terms.te_idxs_woFNs_addFNs(i,:) = go_terms.te_idxs(i,:);
    end

    save(save_name,'go_terms');
    
    disp('GO-term ready');
    
    toc
    
end

fprintf('\n\nFND Ready! The new go-terms data was saved at: %s\n',save_name)
toc(t);
