% Script to run DLS cross-validation test over the go-terms defined by
% Get_GOTerms_Data_Script.m

%% addpaths

addpath ../Functions/DLS_Core/
addpath ../Functions/GO_terms/
addpath ../Functions/Evaluation/
addpath ../Functions/PrintsResults/
addpath ../Data

%% define parameters
test_name = 'FND_DLS'; results.params.test_name = test_name;
directory = '../Results/Expert/';
if ~exist(directory,'dir')
    mkdir(directory);
end

terms_data_dir = '../Data/';
name = 'Bioinf_paper';
year = 2008;
min_annots = 30;
max_annots = 500;
cv_nfolds = 10;
n_cvreps = 3;
 
dw = 3; results.params.dw = dw;
pFN = 0; results.params.pFN = pFN;
conf_th = 0.5; results.params.conf_th = conf_th;
min_feats = 5; results.params.min_feats = min_feats;
w1_feats_th_prctile = 30; results.params.w1_feats_th_prctile = w1_feats_th_prctile;
min_w1 = 0.6; results.params.min_w1 = min_w1;
sigma = 0.1; results.params.sigma = sigma;
min_coexp = 0.6; results.params.min_coexp = min_coexp;
FDR_th = 0.1; results.params.FDR_th = FDR_th;

min_FN_coexp = 0.4; results.params.min_FN_coexp = min_FN_coexp;
sign_min_score = 0; results.params.sign_min_score = sign_min_score;
precW = 2; results.params.precW = precW;
p_dls_preds = 0; results.params.p_dls_preds = p_dls_preds;

use_fns_discovery = true; results.params.use_fns_discovery = use_fns_discovery;
add_FNs = true; results.params.add_FNs = add_FNs;

parallel = true;

%% load data

tic
disp('Loading data...')

% load expression data
load ExpData_expert.mat

% load go-terms data
load(sprintf('%sGO_Terms_Data_year%d_min%d_max%d_cvnfolds%d_%s.mat',terms_data_dir,year,min_annots,max_annots,cv_nfolds,name))
results.go_terms = go_terms;
clear go_terms

toc

%% Run method

t = tic;
disp('Starting cross-validation analisys...')

n_terms = numel(results.go_terms.id);

if isfield(results,'F')
    i_start = find(results.F(:,n_cvreps) == 0,1,'first');
else
    i_start = 1;
    results.conf_th = zeros(n_terms,n_cvreps);
    results.F = zeros(n_terms,n_cvreps);
    results.precision = zeros(n_terms,n_cvreps);
    results.recall = zeros(n_terms,n_cvreps);
    results.preds.confs = cell(n_terms,n_cvreps);
    results.preds.fps_agis = cell(n_terms,n_cvreps);
    results.preds.tps_agis = cell(n_terms,n_cvreps);
    if ~isfield(results.go_terms,'discover_FNs')
        results.go_terms.discover_FNs = false;
    end
end

for i = i_start:3:n_terms
    
    tic
    
    for j = 1:n_cvreps
        
        if use_fns_discovery
            if results.go_terms.discover_FNs
                class0_agis = results.go_terms.class0_agis_woFNs{i};
                nclass0 = results.go_terms.nclass0_woFNs(i);
                tr_idxs = results.go_terms.tr_idxs_woFNs{i,j};
                te_idxs = results.go_terms.te_idxs_woFNs{i,j};
                if add_FNs
                    class1_agis = results.go_terms.class1_agis_wFNs{i};
                    nclass1 = results.go_terms.nclass1_wFNs(i);
                    tr_idxs = results.go_terms.tr_idxs_woFNs_addFNs{i,j};
                    te_idxs = results.go_terms.te_idxs_woFNs_addFNs{i,j};
                end
            else
                error('discover_FNs data not available in results.go_terms')
            end
        else
            class0_agis = results.go_terms.class0_agis{i};
            nclass0 = results.go_terms.nclass0(i);
            class1_agis = results.go_terms.class1_agis{i};
            nclass1 = results.go_terms.nclass1(i);
            tr_idxs = results.go_terms.tr_idxs{i,j};
            te_idxs = results.go_terms.te_idxs{i,j};
        end
        
        eval_terms_results = evalTermsDiffExp(ExpData.DiffExp_FDR01,ExpData.genes_agis,{class1_agis},30);
        results.go_terms.mean_numDiff(i) = eval_terms_results.mean_numDiff;
        
        fprintf('\nGO:%d, name: %s\nnclass1: %d, nclass0: %d\nmean_numDiff: %.2f\nReplicate: %d\n',results.go_terms.id(i),results.go_terms.name{i},nclass1,nclass0,results.go_terms.mean_numDiff(i),j);
    
        [X,Y,genes_agis_X,pvals_X] = makeXY(ExpData.LogRatio,class1_agis,class0_agis,ExpData.genes_agis,ExpData.FDR);
        nFN = ceil(nclass0*pFN/100);
        
        % Run cross-validation for current GO-term
        results_dDLSM_crossval = DLS_CrossVal(X,Y,genes_agis_X,pvals_X,FDR_th,cv_nfolds,dw,min_feats,w1_feats_th_prctile,min_w1,sign_min_score,sigma,precW,nFN,min_FN_coexp,parallel,conf_th,min_coexp,p_dls_preds,tr_idxs,te_idxs);

        % Gather results
        results.conf_th(i,j) = conf_th;
        results.F(i,j) = results_dDLSM_crossval.F;
        results.precision(i,j) = results_dDLSM_crossval.precision1;
        results.recall(i,j) = results_dDLSM_crossval.recall1;
        results.preds.confs{i,j} = results_dDLSM_crossval.preds.max_confs;
        results.preds.fps_agis{i,j} = results_dDLSM_crossval.preds.genes_agis(results_dDLSM_crossval.preds.max_confs > conf_th & results_dDLSM_crossval.Y == -1);
        results.preds.tps_agis{i,j} = results_dDLSM_crossval.preds.genes_agis(results_dDLSM_crossval.preds.max_confs > conf_th & results_dDLSM_crossval.Y == 1);

        fprintf('\nRep%d Done! Max F:%.4f, Precision:%.3f, Recall:%.3f, conf_th:%.4f\n',j,results.F(i,j),results.precision(i,j),results.recall(i,j),results.conf_th(i,j))
    end
    
    printCVResults(sprintf('%sResults_CV_%s.txt',directory,test_name),results,i);
    
    save(sprintf('%sResults_CV_%s.mat',directory,test_name),'results')
    
    toc
    
end

disp('Ready!!')
toc(t);
